@@ -1,34 +1,17 @@
+#setOldClass("zoo")
 setOldClass(c("zooreg", "zoo"))
+setOldClass("xts")
+setClassUnion("xtsORzoo", c("xts","zoo")) # see quantmod
 
-## TODO: This needs to be defined early so that testthat::test()() can unload and reload the
-##       package during development. (apparently, it can't unload everything from `methods'.)
-##
-
-## initially "numeric" was also here but it is hardly a good idea.
-##        TODO: note: if `z' is "mts" then  is(z, "AnyTimeSeries") gives  TRUE
-##        TODO: change SimpleCycle to BareCycle where no info about the seasons is available!
-##
-setClassUnion("AnyTimeSeries", c("ts", "zooreg", "zoo")) ## TODO: add "timeSeries" ?
+setOldClass("Date")
+setOldClass(c("POSIXct", "POSIXt"))
+setOldClass(c("POSIXlt", "POSIXt"))
+setClassUnion("AnyDateTime", c("POSIXct", "POSIXlt", "Pctime", "Date"))
 
 ## virtual class for signatures; all periodic time series classes are its descendants
 ##
 setClass("PeriodicTimeSeries", contains = c("Cyclic", "VIRTUAL") )
 
-## setClassUnion below gives the following error (tested on Lenovo B570):
-##        > devtools::test()
-##        Loading pcts
-##        Testing pcts
-##        ...............................1
-##        1. Error: the new periodic classes are ok --------------------------------------
-##        node stack overflow
-##
-## `R CMD check ' gives the same when running the tests.  Haven't checked if this is bound to
-## devtools but the command seems dodgy, since some of the derived classes from
-## "PeriodicTimeSeries" inherit from "ts", so comment it out.
-##
-## setClassUnion("TSWithRegularTime",
-##               c("PeriodicTimeSeries", "ts", "zooreg")) ## TODO: add "timeSeries" ?
-
 ## basic "native" classes provided by this package
 setClass("PeriodicTS",
          contains = c("PeriodicTimeSeries", "numeric")
@@ -111,11 +94,6 @@ setMethod("initialize", signature(.Object = "PeriodicMTS_ts"),
           }
           )
 
-
-
-## The automatically generated method for as() since it calls new() with no arguments, so the
-## nseasons slot is not set properly:
-##
 ##      > getMethod("coerce", c("ts", "PeriodicTS_ts"))
 ##      Method Definition:
 ##
@@ -127,12 +105,6 @@ setMethod("initialize", signature(.Object = "PeriodicMTS_ts"),
 ##      }
 ##      <environment: namespace:methods>
 ##
-##      Signatures:
-##              from to
-##      target  "ts" "PeriodicTS_ts"
-##      defined "ts" "PeriodicTS_ts"
-##
-##
 ##      > showMethods("coerce", classes = "PeriodicTS_ts", includeDefs = TRUE)
 ##      Function: coerce (package methods)
 ##      from="ts", to="PeriodicTS_ts"
@@ -143,20 +115,26 @@ setMethod("initialize", signature(.Object = "PeriodicMTS_ts"),
 ##          obj
 ##      }
 
-setAs("ts", "PeriodicTS", function(from){ new("PeriodicTS", from) } )
-setAs("ts", "PeriodicMTS", function(from){ new("PeriodicMTS", from) } )
+setAs("ts", "PeriodicTS", function(from){ pcts(from) } )
+setAs("ts", "PeriodicMTS",
+      function(from){
+          wrk <- pcts(from)
+          new("PeriodicMTS", as(wrk, "Cyclic"), matrix(wrk@.Data, ncol = 1))
+      }
+      )
 setAs("ts", "PeriodicTS_ts", function(from){ new("PeriodicTS_ts", from) } )
 setAs("ts", "PeriodicMTS_ts", function(from){ new("PeriodicMTS_ts", from) } )
 
-setAs("mts", "PeriodicMTS", function(from){ new("PeriodicMTS", from) } )
-setAs("mts", "PeriodicTS", function(from){ new("PeriodicMTS", from) } )
-
-## TODO: Conversions between PeriodicXXX classes?
-
-## as.data.frame.pcTimeSeries <- function(x, ...){
-##     as.data.frame(coreMatrix(x))
-## }
-##
+setAs("mts", "PeriodicMTS", function(from){ pcts(from) } )
+setAs("mts", "PeriodicTS",
+      function(from){
+          res <- pcts(from)
+          if(nVariables(from) > 1)
+              stop("the time series is multivariate")
+          ## maybe never will come here, ts() gives "ts" class if there is only one time series
+          res[[1]]
+      }
+      )
 
 setAs("PeriodicTS", "ts", 
       function(from){
@@ -168,22 +146,250 @@ setAs("PeriodicMTS", "ts",
       })
 as.ts.PeriodicTimeSeries <- function(x, ...) as(x, "ts")
 
+## setAs("PeriodicTimeSeries", "Cyclic", 
+##       function(from){
+##           new("Cyclic", cycle = from@cycle, pcstart = from@pcstart)
+##       })
+setAs("PeriodicTS", "Cyclic", 
+      function(from){
+          new("Cyclic", cycle = from@cycle, pcstart = from@pcstart)
+      })
+
+setAs("PeriodicMTS", "Cyclic", 
+      function(from){
+          new("Cyclic", cycle = from@cycle, pcstart = from@pcstart)
+      })
+
+## as.data.frame.pcTimeSeries <- function(x, ...){
+##     as.data.frame(coreMatrix(x))
+## }
+##
+
 setMethod("pcCycle",  "Cyclic", function(x, type, ...) x@cycle)
 
-setMethod("pcCycle",  "PeriodicTimeSeries", function(x, type, ...) x@cycle)
+setMethod("pcCycle",  c(x = "PeriodicTimeSeries", type = "missing"), 
+    function(x, type, ...) x@cycle)
+setMethod("pcCycle",  c(x = "PeriodicTimeSeries", type = "character"), 
+    function(x, type, ...) x@cycle)
 
-setMethod("pcCycle",  "ts",
+setMethod("pcCycle",  c(x = "ts", type = "missing"),
           function(x, type, ...){
               nseasons <- frequency(x)
-              switch(as.character(nseasons),
-                     ## TODO: treat specially other frequencies with builtin classes?
-                     "4"  = new("QuarterYearCycle"),
-                     "12" = new("MonthYearCycle"),
-                     new("BareCycle", nseasons = nseasons)
-                     )
+              BuiltinCycle(nseasons, stop = FALSE)
           }
           )
 
+setMethod("pcCycle",  c(x = "ts", type = "character"),
+          function(x, type, ...){
+              nseasons <- frequency(x)
+              res <- BuiltinCycle(nseasons, stop = FALSE)
+              if(type == "")
+                type <- "BareCycle"
+              as(res, type)
+          }
+          )
+
+setGeneric("pcts", function(x, nseasons, start, ..., keep = FALSE){ standardGeneric("pcts") },
+           signature = c("x", "nseasons") )
+
+.pcts_finalize <- function(x, start, ...){
+    if(missing(start))
+        return(x)
+    if(is.numeric(start) && length(start) == 2){
+        x@pcstart <- start
+    }else{
+        ## TODO: !!! this may need the cycle in some cases
+        date <- as_datetime(start) # as.POSIXct(start) # 2020-04-15 was: as.POSIXlt() # 2020-04-14 was: as.Date()
+        x@pcstart <- .cycle_and_time2pair(x@cycle, date)
+    }
+    x
+}
+
+setMethod("pcts", c(x = "numeric", nseasons = "missing"),
+          function(x, nseasons, start, ...){
+              ## `x' should have  method for frequency() in this case
+              ## NOTE: frequency() has a default method! (returns 1).
+              ## TODO: maybe give warning or error if frequency is 1.
+
+              ##  as.integer() since we don't accept fractional number of seasons
+              nseasons <- as.integer(frequency(x))
+              ## frequency() has a default method which returns 1
+              if(nseasons <= 1)
+                  stop("nseasons is missing and cannot be inferred")
+              pcts(x, nseasons, start, ...)
+          }
+          )
+
+setMethod("pcts", c(x = "matrix", nseasons = "missing"),
+          function(x, nseasons, start, ...){
+              ## `x' should have  method for frequency() in this case
+              ##     see ote in the method for x = "numeric" above
+              nseasons <- as.integer(frequency(x))
+              if(nseasons <= 1)
+                  stop("nseasons is missing and cannot be inferred")
+              pcts(x, nseasons, start, ...)
+          }
+          )
+
+
+setMethod("pcts", c(x = "numeric", nseasons = "numeric"),
+          function(x, nseasons, start, ...){
+              period <- new("SimpleCycle", nseasons = nseasons)
+              wrk <- new("PeriodicTS", cycle = period, x)
+              .pcts_finalize(wrk, start, ...)
+          }
+          )
+
+setMethod("pcts", c(x = "matrix", nseasons = "numeric"),
+          function(x, nseasons, start, ...){
+              period <- new("SimpleCycle", nseasons = nseasons)
+              wrk <- new("PeriodicMTS", cycle = period, x)
+              .pcts_finalize(wrk, start, ...)
+          }
+          )
+
+## todo: check if methods for nseasons "numeric" and "missing" are needed here
+setMethod("pcts", "data.frame",
+          function(x, nseasons, start, ...){
+              pcts(as.matrix(x), nseasons, start, ...)
+          }
+          )
+
+.guess_zoo_cycle <- function(x){ # x must inherit from zoo
+    ## TODO: incomplete
+
+    if(!is.regular(x))
+        ## TODO: can this check give TRUE if times are not monotone?
+        stop("currently pcts requires regular time intervals")
+    ## ## frequency.zoo tries to compute the frequency even if the attribute is not set
+    ## nseasons <- frequency(x)
+    ## if(frequency > 1) could still be inferred
+    index <- index(x)
+    if(is.Date(index)){
+        ## assume initially day of week
+        pct <- Pctime(index, BuiltinCycle(7))
+        ## !!! TODO: this needs cycle.Pctime
+        cyc <- cycle(pct)
+        cycle <- if(all(1:7 %in% unique(cyc)))
+                     BuiltinCycle(7)
+                 else
+                     new("PartialCycle", orig = BuiltinCycle(7), 
+                         subindex = as.integer(sort(unique(cyc))))
+        Cyclic(cycle, start = pct[[1]])
+    }else{
+        stop("This branch not implemented yet - please contact the maintainer of package 'pcts'")
+    }
+}
+
+setMethod("pcts", c(x = "xtsORzoo", nseasons = "missing"),
+          function(x, nseasons, start, ...){
+              ## ignore argument start for now
+
+              cyclic <- .guess_zoo_cycle(x)
+              start <- stats::start(cyclic) 
+              cycle <- cyclic@cycle
+
+              nseasons <- nSeasons(cycle)
+              nseas <- if(is(cycle, "PartialCycle"))
+                           nSeasons(cycle@orig)
+                       else
+                           nseasons
+
+              x_ts <- as.ts(as.zooreg(x))
+              if(nseas == nseasons){
+                  pcts(x_ts, cycle, start = start, ...)
+              }else{
+                  res <- pcts(x_ts, cycle@orig, start = start, ...)
+                  window(res, seasons = cycle@subindex)
+              }
+         } 
+         )
+
+.ts2periodic_ts <- function(x, cls, nseasons, ...){
+    cyc <- if(!missing(nseasons)  &&  frequency(x) != nseasons){
+               ## frequency(x) <- nseasons NOTE: no "ts" method for "frequency<-"
+               pcCycle(nseasons) # creates a bare cycle
+           }else{
+               pcCycle(x)
+           }
+    new(cls, x, cycle = cyc, pcstart = start(x))
+}
+
+setMethod("pcts", c(x = "ts", nseasons = "missing"),
+          function(x, nseasons, start, ..., keep){
+              wrk <- if(keep)
+                         new("PeriodicTS_ts", x)
+                     else
+                         .ts2periodic_ts(x, "PeriodicTS", nseasons)
+              .pcts_finalize(wrk, start, ...)
+          }
+          )
+
+setMethod("pcts", c(x = "ts", nseasons = "numeric"),
+          function(x, nseasons, start, ..., keep){
+              wrk <- if(keep){
+                         if(frequency(x) != nseasons)
+                             stop("please change the frequency of the ts object or use keep = FALSE")
+                         new("PeriodicTS_ts", x)
+                     }else
+                         .ts2periodic_ts(x, "PeriodicTS", nseasons)
+              .pcts_finalize(wrk, start, ...)
+          }
+          )
+
+setMethod("pcts", c(x = "mts", nseasons = "missing"),
+          function(x, nseasons, start, ..., keep){
+              wrk <- if(keep)
+                         new("PeriodicMTS_ts", x)
+                     else
+                         .ts2periodic_ts(x, "PeriodicMTS", nseasons)
+              .pcts_finalize(wrk, start, ...)
+          }
+          )
+
+setMethod("pcts", c(x = "mts", nseasons = "numeric"),
+          function(x, nseasons, start, ..., keep){
+              wrk <- if(keep){
+                         if(frequency(x) != nseasons)
+                             stop("please change the frequency of the ts object or use keep = FALSE")
+                         new("PeriodicMTS_ts", x)
+                     }else
+                         .ts2periodic_ts(x, "PeriodicMTS", nseasons)
+              .pcts_finalize(wrk, start, ...)
+          }
+          )
+
+setMethod("pcts", c(x = "numeric", nseasons = "BasicCycle"),
+          function(x, nseasons, start, ...){
+              wrk <- new("PeriodicTS", cycle = nseasons, x)
+              .pcts_finalize(wrk, start, ...)
+          }
+          )
+
+setMethod("pcts", c(x = "matrix", nseasons = "BasicCycle"),
+          function(x, nseasons, start, ...){
+              wrk <- new("PeriodicMTS", cycle = nseasons, x)
+              .pcts_finalize(wrk, start, ...)
+          }
+          )
+
+## base R
+##
+## cycle() gives the season (a number)
+## time() gives the time, say 2014.75 for the 3rd quarter of 2014
+## frequency() gives the number of seasons
+## deltat()
+## start() end() (but these are for compatibility with S2 only)
+## window()
+
+## zoo
+##
+## index(), time() - times of the observations; start(), end()
+## coredata() - gives a plain vector/matrix
+## merge() - union and intersection
+## plot()
+## window()
+
 frequency.PeriodicTimeSeries <- function(x, ...) nSeasons(x)
 deltat.PeriodicTimeSeries    <- function(x, ...) 1 / nSeasons(x)
 cycle.PeriodicTimeSeries     <- function(x, ...){
@@ -207,98 +413,15 @@ time.PeriodicTimeSeries <- function(x, offset = 0, ...){
     new("PeriodicTS", as(x, "Cyclic"), res)
 }
 
-setGeneric("pcts", def = function(x, nseasons, keep = FALSE, ...){ standardGeneric("pcts") })
-
-## TODO: add further methods for pcts
-
-
-setMethod("pcts", "numeric",
-          function(x, nseasons, keep, ...){
-              if(missing(nseasons)) # `x' should have  method for frequency() in this case
-                  nseasons <- as.integer(frequency(x))
-              period <- new("SimpleCycle", nseasons = nseasons)
-              new("PeriodicTS", cycle = period, x, ...)
-          }
-          )
-
-setMethod("pcts", "matrix",
-          function(x, nseasons, keep, ...){
-              if(missing(nseasons)) # `x' should have  method for frequency() in this case
-                                    # note: frequency() has a default method!
-                  nseasons <- as.integer(frequency(x))
-              period <- new("SimpleCycle", nseasons = nseasons)
-              new("PeriodicMTS", cycle = period, x, ...)
-          }
-          )
-
-.ts2periodic_ts <- function(x, cls, nseasons){
-    if(!missing(nseasons)  &&  frequency(x) != nseasons){
-        ## frequency(x) <- nseasons NOTE: no "ts" method for "frequency<-"
-        cyc <- pcCycle(nseasons) # creates a bare cycle
-    }else{
-        cyc <- pcCycle(x)
-    }
-    new(cls, x, cycle = pcCycle(x), pcstart = start(x))
-}
-
-setMethod("pcts", "ts",
-          function(x, nseasons, keep, ...){
-              if(keep){
-                  if(!missing(nseasons)  &&  frequency(x) != nseasons)
-                      stop("please change the frequency of the ts object or use keep = FALSE")
-                  new("PeriodicTS_ts", x)
-              }else{
-                  .ts2periodic_ts(x, "PeriodicTS", nseasons)
-              }
-          }
-          )
-
-setMethod("pcts", "mts",
-          function(x, nseasons, keep, ...){
-              if(keep){
-                  if(!missing(nseasons)  &&  frequency(x) != nseasons)
-                      stop("please change the frequency of the ts object or use keep = FALSE")
-                  new("PeriodicMTS_ts", x)
-              }else{
-                  # 2019-04-19 was: new("SimpleCycle", nseasons = as.integer(frequency(x)))
-                  .ts2periodic_ts(x, "PeriodicMTS", nseasons)
-              }
-          }
-          )
-
-## TODO: add further subclasses of "pcTimeSeries" for work with other time series classes.
-
-## base R
-##
-## cycle() gives the season (a number)
-## time() gives the time, say 2014.75 for the 3rd quarter of 2014
-## frequency() gives the number of seasons
-## deltat()
-## start() end() (but these are for compatibility with S2 only)
-## window()
-
-## zoo
-##
-## index(), time() - times of the observations; start(), end()
-## coredata() - gives a plain vector/matrix
-## merge() - union and intersection
-## plot()
-## window()
-
-
-## setMethod("show","pcTs1",
-##           function(object){
-##           # y <- ts(data=pc.data.vec(object@.pcdata),frequency=pc.nseasons(object@.pcdata))
-##             pc.boxplot(object)
-##             callNextMethod()
-##           }
-##           )
-
 setMethod("show", "PeriodicTS",
           function(object){
-              ## y <- ts(data=pc.data.vec(object@.pcdata),frequency=pc.nseasons(object@.pcdata))
-              ## pc.boxplot(object)
-              ##  callNextMethod()
+              .reportClassName(object, "PeriodicTS")
+              ## show(as(object, "Cyclic"))
+              cat("Slot \"cycle\": ")
+              ## show(object@cycle)
+              show(as(object, "Cyclic"))
+              cat("\n")
+              
               start <- start(object)
               end   <- end(object)
               nseas <- nSeasons(object)
@@ -306,6 +429,8 @@ setMethod("show", "PeriodicTS",
               wrk <- seq(start(object)[1], end(object)[1])
               wrk2 <- rep(wrk, each = nSeasons(object))
               cycles <- wrk2[seq(start(object)[2], length = nTicks(object))]
+              cycles_prefix <- substring(unitCycle(object), 1, 1)
+
               cyc <- cycle(object)
 
               data <- object@.Data
@@ -315,37 +440,50 @@ setMethod("show", "PeriodicTS",
                   data <- c(data, rep(NA_real_, nseas - end[2]))
 
               data <- object@.Data
-              if(length(data) %% nseas == 0){
+              if(length(data) %% nseas == 0  && start[2] == 1){
                   data <- matrix(data, ncol = nSeasons(object), byrow = TRUE)
-                  rownames(data) <- as.character(start[1]:end[1])
+                  rownames(data) <- paste0(cycles_prefix, start[1]:end[1])
                   ## TODO: sort out the method for "Cyclic" to work with abb = TRUE
                   ##     colnames(data) <- allSeasons(object, abb = TRUE)
                   colnames(data) <- allSeasons(object@cycle, abb = TRUE)
+                  print(data)
               }else{
-                  ## krapka
-                  ## TODO: it is confusing to have different output,
-                  ##       maybe do it always this way?
-                  names(data) <- paste0(cycles, "_", cyc)
+                  data <- c(rep(NA_real_, start[2] - 1), data, rep(NA_real_, nseas - end[2]))
+                  data <- matrix(data, ncol = nSeasons(object), byrow = TRUE)
+                  wrk <- format(data)
+                  wrk[1, seq_len(start[2] - 1)] <- ""
+                  if(end[2] < nseas)
+                      wrk[nrow(wrk), (end[2] + 1) : nseas] <- ""
+                  rownames(wrk) <- paste0(cycles_prefix, start[1]:end[1])
+                  colnames(wrk) <- allSeasons(object@cycle, abb = TRUE)
+                  print(wrk, quote = FALSE)
               }
-              print(data)
-              ## show(as(object, "Cyclic"))
           }
           )
 
+## An object of class "Cyclic"
+## Slot "cycle":
+## Object from built-in class 'MonthYearCycle'
+## Cycle start: January
+
 setMethod("show", "PeriodicMTS",
           function(object){
-              ## y <- ts(data=pc.data.vec(object@.pcdata),frequency=pc.nseasons(object@.pcdata))
-              ## pc.boxplot(object)
-              ##  callNextMethod()
+              .reportClassName(object, "PeriodicMTS")
+              ## show(as(object, "Cyclic"))
+              cat("Slot \"cycle\": ")
+              show(object@cycle)
+              cat("\n")
+
               wrk <- seq(start(object)[1], end(object)[1])
               wrk <- rep(wrk, each = nSeasons(object))
               cycles <- wrk[seq(start(object)[2], length = nTicks(object))]
+              cycles_prefix <- substring(unitCycle(object), 1, 1)
+
               cyc <- cycle(object)
 
               data <- object@.Data
-              rownames(data) <- paste0(cycles, "_", cyc)
+              rownames(data) <- paste0(cycles_prefix, cycles, "_", cyc)
               print(data)
-              show(as(object, "Cyclic"))
           }
           )
 
@@ -405,9 +543,13 @@ boxplot.PeriodicTimeSeries <- function(x, ...){
     invisible(res)
 }
 
-nTicks <- function(x, ...)
-    NROW(x)
-setGeneric("nTicks")
+# nTicks <- function(x, ...)
+#     NROW(x)
+setGeneric("nTicks", function(x){ standardGeneric("nTicks") })
+setMethod("nTicks", "numeric", function(x) length(x) )
+setMethod("nTicks", "matrix", function(x) nrow(x) )
+setMethod("nTicks", "PeriodicTimeSeries", function(x) NROW(x) )
+setMethod("nTicks", "Cyclic", function(x) 1 ) ## todo: currently always one.
 
 nVariables <- function(x, ...)
     NCOL(x)
@@ -463,10 +605,6 @@ pcArray <- function(x, ndim = 3, ...){ # ndim not used in the default method
 }
 setGeneric("pcArray")
 
-## removing: doesn't make sense to have this simply for the transpose of 
-##     tsMatrix() and at the moment I don't see a natural definition analogue 
-##     of pcMatrix() for the ts case.
-##
 ## pctsMatrix <- function(x, ...){
 ##     t(pcMatrix(x, ...))
 ## }
@@ -479,12 +617,24 @@ pctsArray <- function(x, ndim = 3, ...){ # ndim not used in the default method
 }
 setGeneric("pctsArray")
 
+setMethod("[", c(x = "PeriodicTS", i = "missing", j = "missing"), 
+          function(x){
+              x@.Data
+          })
+
+setMethod("[", c(x = "PeriodicTS", i = "AnyDateTime", j = "missing"), 
+          function(x, i){
+              ## TODO: is this reliable?
+              ind <- which(as_datetime(x)  %in% as_datetime(i))
+              x@.Data[ind]
+          })
+
 setMethod("[[", c(x = "PeriodicMTS"), 
           function(x, i){
               if (length(i) != 1) 
-                  ## call. = FALSE here since otherwise the error messaage starts
+                  ## call. = FALSE here since otherwise the error message starts
                   ## with "Error in .local(x, i, ...) :" which is non-informative.
-                  stop("in '[[' length of argument 'i' must be equal to one", 
+                  stop("for [[ the length of argument i must be equal to one", 
                        call. = FALSE)
               new("PeriodicTS", as(x, "Cyclic"), x@.Data[ , i])
           })
@@ -500,8 +650,10 @@ setMethod("[", c(x = "PeriodicMTS", i = "ANY", j = "missing"),
               if(nposargs(sys.call()) == 2) # x[i]
                   new("PeriodicMTS", as(x, "Cyclic"), x@.Data[ , i, drop = FALSE])
               else{ # x[i, ]
-                  ## x@.Data[i, , ...]
-                  stop("use x[][i, ] or x[][i,j] if you wish to use matrix indexing")
+                  ## 2020-04-19: allowing matrix indexing
+                  ## stop("use x[][i, ] or x[][i,j] if you wish to use matrix indexing")
+                  j <- 1:ncol(x@.Data)
+                  x@.Data[i, j, ...]
               }
           })
 
@@ -518,32 +670,35 @@ setMethod("[", c(x = "PeriodicMTS", i = "missing", j = "missing"),
 ##               x@.Data[ , , ...]
 ##           })
 
-pctime2ind <- function(x, start, nseasons){
-    (x[1] - start[1]) * nseasons + (x[2] - start[2]) + 1
-}
+setMethod("[", c(x = "PeriodicMTS", i = "ANY", j = "ANY"), 
+          function(x, i, j, ...){
+              x@.Data[i, j, ...]
+          })
 
-ind2pctime <- function(x, start, nseasons){
-    wrk <- x %% nseasons
-    if(wrk == 0)
-        wrk <- nseasons
-    res <- c(start[1] + (x - 1) %/% nseasons, start[2] + wrk - 1)
-    if(res[2] > nseasons)
-                     # or more robustly: 
-                     #    res + c(res[2] %/% nseasons, res2 %% nseasons)
-        res + c(1,  res[2] - nseasons)
-    else
-        res
-}
+setMethod("[", c(x = "PeriodicMTS", i = "AnyDateTime", j = "missing"), 
+          function(x, i){
+              ## TODO: is this reliable?
+              ind <- which(as_datetime(x)  %in% as_datetime(i))
+              x@.Data[ind, ]
+          })
+
+setMethod("[", c(x = "PeriodicMTS", i = "AnyDateTime", j = "ANY"), 
+          function(x, i, j){
+              ## TODO: is this reliable?
+              ind <- which(as_datetime(x)  %in% as_datetime(i))
+              x@.Data[ind, j]
+          })
 
 start.Cyclic <- function(x, ...){
     x@pcstart
 }
 
-end.PeriodicTimeSeries <- function(x, ...){
+## end.PeriodicTimeSeries 
+end.Cyclic <- function(x, ...){
     ind2pctime(nTicks(x), x@pcstart, nSeasons(x))
 }
 
-window.PeriodicTS <- function(x, start = NULL, end = NULL, ...){
+window.PeriodicTS <- function(x, start = NULL, end = NULL, seasons = NULL, ...){
     nseas <- nSeasons(x)
     time1st <- x@pcstart
     begind <- if(is.null(start))
@@ -560,10 +715,47 @@ window.PeriodicTS <- function(x, start = NULL, end = NULL, ...){
     if(!is.null(start))
         cyc@pcstart <- start
 
-    new("PeriodicTS", cyc, .Data = x@.Data[begind:endind])
+    if(!is.null(seasons)){
+        ## TODO: check validity of 'seasons'
+        lind <- logical(nTicks(x))
+        lind[begind:endind] <- TRUE
+        lind <- lind & (cycle(x) %in% seasons)
+
+        wrkdata <- x@.Data[lind]
+
+        ind.1st <- which(lind)[1]
+        newstart <- ind2pctime(ind.1st, start(x), nseas)
+        newstart[2] <- which(seasons == newstart[2])
+
+        ## TODO: more is needed here
+        cy <- ## if(is(x@cycle, "DayWeekCycle"))
+              ##     new("PartialDayWeekCycle", subindex = seasons)
+              ## else
+                  new("PartialCycle", orig = x@cycle, subindex = seasons) 
+#browser()
+        ## ## TODO: doesn't work, apparently "Cyclic" needs to pass on .Data
+        ## ##     res <- new("PeriodicTS", cycle = cy, pcstart = newstart, .Data = wrkdata)
+        ## ## so do it this way:
+        ##     cyc <- new("Cyclic", cycle = cy, pcstart = newstart)
+        ##     res <- new("PeriodicTS", cyc, .Data = wrkdata)
+        ## no, argument .Data seems the culprit;
+        ## this works:
+        ##    new("PeriodicTS", cycle = BareCycle(4), pcstart = c(1, 1), 1:12)
+        ## but this doesn't:
+        ##    new("PeriodicTS", cycle = BareCycle(4), pcstart = c(1, 1), .Data = 1:12)
+        ## so, just give the data unnamed (also below):
+        res <- new("PeriodicTS", cycle = cy, pcstart = newstart, wrkdata)
+        return(res)
+    }
+
+    ## new("PeriodicTS", cyc, .Data = x@.Data[begind:endind])
+    new("PeriodicTS", cyc, x@.Data[begind:endind])
 }
 
-window.PeriodicMTS <- function(x, start = NULL, end = NULL, ...){
+window.PeriodicMTS <- function(x, start = NULL, end = NULL, seasons = NULL, ...){
+    ## this is almost identical to window.PeriodicTS,
+    ##      could be made common:  (1) new is called with  PeriodicTS or PeriodicMTS
+    ##                             (2) access x@.Data with the equialent of coredata functions
     nseas <- nSeasons(x)
     time1st <- x@pcstart
     begind <- if(is.null(start))
@@ -580,7 +772,66 @@ window.PeriodicMTS <- function(x, start = NULL, end = NULL, ...){
     if(!is.null(start))
         cyc@pcstart <- start
 
-    new("PeriodicMTS", cyc, .Data = x@.Data[begind:endind, , drop = FALSE])
+    if(!is.null(seasons)){
+        ## TODO: check validity of 'seasons'
+        lind <- logical(nTicks(x))
+        lind[begind:endind] <- TRUE
+        lind <- lind & (cycle(x) %in% seasons)
+
+        wrkdata <- x@.Data[lind, , drop = FALSE]
+
+        ind.1st <- which(lind)[1]
+        newstart <- ind2pctime(ind.1st, start(x), nseas)
+        newstart[2] <- which(seasons == newstart[2])
+
+        ## TODO: more is needed here
+        cy <- ## if(is(x@cycle, "DayWeekCycle"))
+              ##     new("PartialDayWeekCycle", subindex = seasons)
+              ## else
+                  new("PartialCycle", orig = x@cycle, subindex = seasons) 
+        res <- new("PeriodicMTS", cycle = cy, pcstart = newstart, wrkdata)
+    
+        return(res)
+    }
+
+
+    new("PeriodicMTS", cyc, x@.Data[begind:endind, , drop = FALSE])
+}
+
+`window<-.PeriodicTS` <- function(x, start = NULL, end = NULL, ..., value){
+    nseas <- nSeasons(x)
+    time1st <- x@pcstart
+    begind <- if(is.null(start))
+                  1
+              else
+                  pctime2ind(start, time1st, nseas)
+        
+    endind <- if(is.null(end))
+                  nTicks(x)
+              else
+                  pctime2ind(end, time1st, nseas)
+
+    ## TODO: check lengths?
+    x@.Data[begind:endind] <- value
+    x
+}
+
+`window<-.PeriodicMTS` <- function(x, start = NULL, end = NULL, ..., value){
+    nseas <- nSeasons(x)
+    time1st <- x@pcstart
+    begind <- if(is.null(start))
+                  1
+              else
+                  pctime2ind(start, time1st, nseas)
+        
+    endind <- if(is.null(end))
+                  nTicks(x)
+              else
+                  pctime2ind(end, time1st, nseas)
+
+    ## TODO: check lengths?
+    x@.Data[begind:endind, ] <- value
+    x
 }
 
 setMethod("head", "PeriodicTimeSeries",
@@ -611,24 +862,29 @@ setMethod("tail", "PeriodicTimeSeries",
               window(x, start = start)
           })
 
-availStart <- function(x) UseMethod("availStart")
-availStart.default <- function(x){
+availStart <- function(x, any = TRUE) UseMethod("availStart")
+availStart.default <- function(x, any = TRUE){
     ind <- match(FALSE, is.na(as.vector(x)))
     if(is.na(ind))
         stop("No non-missing values in x")
     ind2pctime(ind, start(x), nSeasons(x))
 }
 
-availStart.matrix <- function(x){
+availStart.matrix <- function(x, any = TRUE){
     m <- as.matrix(x)
-    ind <- min(apply(m, 2, function(obj) match(FALSE, is.na(obj)) ))
+    ind <- if(any)
+               min(apply(m, 2, function(obj) match(FALSE, is.na(obj)) ))
+           else # all
+               match(TRUE, complete.cases(m))
+    
     if(is.na(ind))
-        stop("No non-missing values in x")
+        stop(if(any) "No non-missing values in x" else "No complete cases in x" )
+    
     ind2pctime(ind, start(x), nSeasons(x))
 }
 
-availEnd <- function(x) UseMethod("availEnd")
-availEnd.default <- function(x){
+availEnd <- function(x, any = TRUE) UseMethod("availEnd")
+availEnd.default <- function(x, any = TRUE){
     y <- rev(as.vector(x))
     ind <- match(FALSE, is.na(y))
     if(is.na(ind))
@@ -637,15 +893,42 @@ availEnd.default <- function(x){
     ind2pctime(ind, start(x), nSeasons(x))
 }
 
-availEnd.matrix <- function(x){
+availEnd.matrix <- function(x, any = TRUE){
     m <- as.matrix(x)
-    ind <- min(apply(m, 2, function(obj) match(FALSE, is.na(rev(obj))) ))
+    ## TODO: use complete.cases()
+    ind <- if(any)
+               min(apply(m, 2, function(obj) match(FALSE, is.na(rev(obj))) ))
+           else
+               match(TRUE, rev(complete.cases(m)))
+
     if(is.na(ind))
-        stop("No non-missing values in x")
+        stop(if(any) "No non-missing values in x" else "No complete cases in x" )
+
     ind <- nrow(m) - ind + 1
     ind2pctime(ind, start(x), nSeasons(x))
 }
 
+na.trim.PeriodicTS <- function (object, sides = c("both", "left", "right"), ...){
+    switch(match.arg(sides),
+           both = window(object, start = availStart(object), end = availEnd(object)),
+           left = window(object, start = availStart(object)),
+           right = window(object, end = availEnd(object))
+           )
+}
+
+na.trim.PeriodicMTS <-
+    function (object, sides = c("both", "left", "right"), is.na = c("any", "all"), ...){
+        any <- match.arg(is.na) == "all"
+        sides <- match.arg(sides)
+        if(sides != "right")  start <- availStart(object, any)
+        if(sides != "left")   end <- availEnd(object, any)
+        switch(sides,
+               both = window(object, start = start, end = end),
+               left = window(object, start = start),
+               right = window(object, end = end)
+               )
+    }
+
 setMethod("plot", c(x = "PeriodicTS", y = "missing"),
           function(x, y, main = NULL, ...){
               ## for now just call the base "ts" method
@@ -672,6 +955,40 @@ setMethod("plot", c(x = "PeriodicMTS", y = "missing"),
               }
           })
 
+## as.Date.PeriodicTimeSeries
+setMethod("as_date", "PeriodicTimeSeries",
+          function(x, ...){
+              as_date(as_datetime(x, ...))
+          })
+
+setMethod("as_datetime", "PeriodicTimeSeries",
+          function(x, ...){
+              startdate <- as_datetime(as(x, "Cyclic")) # as.Date(as(x, "Cyclic"))
+
+              n <- nTicks(x)
+              nseas <- nSeasons(x)
+              seasind <- 1:nSeasons(x)
+
+              units <- .get_period_units(x@cycle)
+              plen <- .get_period_length(x@cycle)
+              p <- period(plen, units)
+              cls <- class(x@cycle)
+
+              shiftall <- .cycle_offsets(x@cycle, n, start(x)[2])
+              
+                  # without as.Date it's class is [1] "POSIXct" "POSIXt" 
+                  # as.Date(startdate + p * shiftall)
+              startdate + p * shiftall
+          })
+
+as.POSIXct.PeriodicTimeSeries <- function(x, ...){
+    as_datetime(x, ...)
+}
+
+as.Date.PeriodicTimeSeries <- function(x, ...){
+    as_date(as_datetime(x, ...))
+}
+
 setMethod("summary", c(object = "PeriodicTS"),
           function(object, alwaysNA = TRUE, ...){
               wrk <- summary(object@.Data)
@@ -701,6 +1018,80 @@ setMethod("summary", c(object = "PeriodicMTS"),
               do.call("rbind", wrk)
           })
 
+setMethod("pcMean", signature("numeric"),
+          function(object, nseasons, ...){
+              structure(pc_mean(object, nseasons, ...), 
+                        names = allSeasons(BareCycle(nseasons)))
+          }
+          )
+
+setMethod("pcMean", signature("matrix"),
+          function(object, nseasons, ...){
+              nc <- ncol(object)
+              nr <- nrow(object)
+              if(nr %% nseasons == 0){
+                  dim(object) <- c(nseasons, nr / nseasons, nc)
+                  res <- apply(object, c(1, 3), mean, ...)
+              }else{
+                  res <- sapply(seq_len(nc), function(i) pc_mean(object[ , i], nseasons, ...))
+              }
+              colnames(res) <- colnames(object)
+              rownames(res) <- allSeasons(BareCycle(nseasons))
+              res
+          }
+          )
+
+setMethod("pcMean", signature("PeriodicTS"),
+          function(object, ...){
+              structure(pc_mean(object, nSeasons(object), ...), names = allSeasons(object))
+          }
+          )
+
+setMethod("pcMean", signature("PeriodicMTS"),
+          function(object, ...){
+              nc <- ncol(object)
+              ## nseas <- nSeasons(object)
+              res <- sapply(seq_len(nc), function(i) pcMean(object[[i]], ...))
+              colnames(res) <- colnames(object)
+              res
+          }
+          )
+
+setMethod("pcApply", signature("numeric"),
+          function(object, nseasons, FUN, ...){
+              structure(pc_apply(object, nseasons, FUN, ...),
+                        names = allSeasons(BareCycle(nseasons)))
+          }
+          )
+
+setMethod("pcApply", signature("matrix"),
+          function(object, nseasons, FUN, ...){
+              nc <- ncol(object)
+              ## nseas <- nSeasons(object)
+              res <- sapply(seq_len(nc), 
+                            function(i) pc_apply(object[ , i], nseasons, FUN, ...))
+              colnames(res) <- colnames(object)
+              res
+          }
+          )
+
+setMethod("pcApply", signature("PeriodicTS"),
+          function(object, FUN, ...){
+              structure(pc_apply(object, nSeasons(object), FUN, ...),
+                        names = allSeasons(object))
+          }
+          )
+
+setMethod("pcApply", signature("PeriodicMTS"),
+          function(object, FUN, ...){
+              nc <- ncol(object)
+              ## nseas <- nSeasons(object)
+              res <- sapply(seq_len(nc), function(i) pcApply(object[[i]], FUN, ...))
+              colnames(res) <- colnames(object)
+              res
+          }
+          )
+
 setMethod("autocovariances", signature(x = "PeriodicTS"),
           function(x, maxlag, ...){
               ## TODO: It may make sense to give the user the choice of computing
@@ -739,7 +1130,7 @@ setMethod("autocorrelations", signature(x = "PeriodicTimeSeries", maxlag = "ANY"
                   stop("Method not implemented yet for multivariate time series")
               acv <- autocovariances(x, maxlag = maxlag, ...)
               sd <- sqrt(acv[[0]])
-              fac <- pc.sdfactor(sd, maxlag)[ , 1 + (0:maxlag)] # "1+" since "matrix"
+              fac <- pc_sdfactor(sd, maxlag)[ , 1 + (0:maxlag)] # "1+" since "matrix"
               res <- acv / fac # this assumes that '/' is defined for 'acv'
                                # (which it is for Lagged objects)
                   # 2019-05-14 was:  res # a "Lagged2d" object
